/** C++ Interface: simstream
//
// Description: 
//
//
// Author: Georgios Kioumourtzis <gkioumou@gmail.com>, (C) 2008
//

 * Copyright (c) 2007 University of Patras Computer Engineering and Informatics Department Greece
 *
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "simstream.h"

#define NOW Scheduler::instance().clock()
#define TIMER_TIMEOUT 5
#define STARTUP_TIMEOUT 5
#define RATE_FACTOR 0.7
#define UP_LEVEL_FACTOR 1.2
#define DOWN_LEVEL_FACTOR 0.8

static class SimStreamClass : public TclClass {
public:
	SimStreamClass() : TclClass("SimStream/RTP") {}
	TclObject* create(int, const char*const*) {
		return (new SimStream());
	}
} class_sim_stream;

/**
 * no argument constructor 
 */
SimStream::SimStream()
	: sessions_(0),streamid_(-1)
{
	bind_bool("isSender",&isSender);	
	bind_bool("isReceiver",&isReceiver);
	bind("level_",&level_);
	bind("rcv_rate_",&rcv_rate_);
	inst_rx_rate_ = 0;
	avg_rx_ = 0;
	bind("low_tx_",&low_tx_);
	bind("medium_tx_",&medium_tx_);
	bind("high_tx_",&high_tx_);
	bind("low_threshold_", &low_threshold_);
	bind("medium_threshold_", &medium_threshold_);
	bind("high_threshold_", &high_threshold_);

	
}

/**
 * destructor
*/
SimStream::~SimStream()
{
	while (sessions_ != 0) {
		RTPSession* s = sessions_;
		sessions_ = sessions_->next;
		delete s;
	}
	rx_lst.~list<double>();
	
	
}

/**
 *  enters a session
 * @param s  the session to be added
 */
void SimStream::enter(RTPSession* s)
{
	
	printf("Time:%f Stream:%d ...I add session %d \n", NOW,streamid_ , s->srcid());
	s->next = sessions_;
	sessions_ = s;
	
}


/**
  * the command function that handles TCL commands
 *
 */
int SimStream::command(int argc, const char*const* argv)
{
	if (argc == 3) {
		if (strcmp(argv[1], "session") == 0) {
			RTPSession* s = (RTPSession*)TclObject::lookup(argv[2]);
			enter(s);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "stream-id") == 0) {
			streamid_ = atoi(argv[2]);
			return (TCL_OK);
		}
	}

	return (TclObject::command(argc, argv));
}


/**
 * adds the instantaneius receiving rate to the list
 * @param theValue  the value to be added
 */
void SimStream::setInst_rx_rate(double theValue)
{ 
	avg_rx_ = avg_rx_ * 0.5 + 0.5 * theValue;
	
}

/**
 *  sets the average values of all streams
 * @param theValue  the average transmission rate
 * @param level  the stream level
 */
void SimStream::setAvg_tx(double theValue, int level)
{
	if(level==1) {
		low_tx_ =  theValue;
	} else if(level==2) {
		medium_tx_ = theValue;
	} else high_tx_ = theValue;


}


/**
 * campares the average receiving rate with stream limits and average 
 * transmission rates 
 */
void SimStream::compare()
{

	switch(level_) {
		case 1	:
			
			if(avg_rx_ >= UP_LEVEL_FACTOR * medium_threshold_ && avg_rx_ >= medium_tx_ * RATE_FACTOR) {

			 go_up(level_);
			}
			break;
		case 2	:
		
			if(avg_rx_ <= DOWN_LEVEL_FACTOR * medium_threshold_) {

				go_down(level_);
			} else if(avg_rx_ >= UP_LEVEL_FACTOR * high_threshold_ && avg_rx_ >= high_tx_ * RATE_FACTOR) {

				go_up(level_);
			}
			break;
		case 3	:
	
			if(avg_rx_ <= DOWN_LEVEL_FACTOR * high_threshold_ ) {

			go_down(level_);
			}
			break;
		default	:	
		printf("Error...no level found\n");
	}
		
}

/**
 *  go up one level
 * @param level  the current level
 */
void SimStream::go_up(int level)
{
	Tcl::instance().evalf("%s go-up %d",name(),level);
	printf("Time:%f Stream:%d level:%d I go UP  Avg_rx: %f Avg_Tx_Low:%f Avg_Tx_Medium:%f Avg_Tx_High:%f \n", NOW,streamid_ , level, avg_rx_ * 8, low_tx_ * 8, medium_tx_ * 8, high_tx_ * 8  );
	
}

/**
 * go down one level
 * @param level  the current level
 */
void SimStream::go_down(int level)
{
	Tcl::instance().evalf("%s go-down %d",name(),level);
	printf("Time:%f Stream:%d level:%d I go DOWN  Avg_rx: %f Avg_Tx_Low:%f Avg_Tx_Medium:%f Avg_Tx_High:%f \n", NOW,streamid_ , level, avg_rx_ * 8, low_tx_ * 8, medium_tx_ * 8, high_tx_ * 8  );
}








