/** C++ Interface: simstream
//
// Description: 
//
//
// Author: Georgios Kioumourtzis <gkioumou@gmail.com>, (C) 2008
//
 * Copyright (c) 2007 University of Patras Computer Engineering and Informatics Department Greece
 *
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#include "config.h"
#include "object.h"
#include "agent.h"
#include "timer-handler.h"
#include <list>	
#include "rtp.h"

class RTPSession;
class SimStream ;

class SimStream : public TclObject {
public:
	SimStream();
	~SimStream();
	SimStream* next;
	RTPSession* sessions_;
	/* connection with TCL commands */
	int command(int argc, const char*const* argv);
	int get_streamid() {return streamid_;}
	void setLevel ( int theValue )	{level_ = theValue;}
	int getLevel() {return level_;}
	void setRcv_rate ( double theValue ) {rcv_rate_ = theValue;}
	double getRcv_rate() {	return rcv_rate_;}
	void setInst_rx_rate( double theValue );
	double getInst_rx_rate() { return inst_rx_rate_;}
	void setAvg_rx ( double theValue ) {avg_rx_ = theValue;	}
	double getAvg_rx() {return avg_rx_;}
	void setAvg_tx ( double theValue, int level);
	void setLow_tx(double theValue) {low_tx_ = theValue;}
	void setMedium_tx(double theValue) {medium_tx_ = theValue;}
	void setHigh_tx(double theValue) {high_tx_=theValue;}
	double getLow_tx() {return low_tx_;}
	double getMedium_tx() {	return medium_tx_;}
	double getHigh_tx() {return high_tx_;}
	void compare();
	
	
	
		

protected:
	void enter(RTPSession*);
	/** the stream id */
	int streamid_;
	/** binded from tcl script */
	int isSender;
	/** binded from the tcl script */
	int isReceiver;
	/** the level of the stream. see definition in ns-default.tcl */
	int level_;
	/** the receiving rate */
	double rcv_rate_;
	/** instantaneous receiving rate */
	double inst_rx_rate_;
	/** a list to keep all calculating TCP-friendly estimations over Δt*/
	list<double> rx_lst;
	/** the average receiving rate */
	double avg_rx_;
	/** transmission rate of the low capacity stream */
	double low_tx_;
	/** transmission rate of the medium capacity stream */
	double medium_tx_;
	/** transmission rate of the high capacity stream */
	double high_tx_;
	void go_up(int level);
	void go_down(int level);
	/** the low linit of the low capacity stream */ 
	double low_threshold_;
	/** the low linit of the medium capacity stream */ 
	double medium_threshold_;
	/** the low linit of the high capacity stream */ 
	double high_threshold_;
};




